xquery version "3.1";

module namespace search="http://grammateus.ch/search";

import module namespace templates="http://exist-db.org/xquery/templates" ;
import module namespace app="http://grammateus.ch/templates" at "app.xql";
import module namespace config="http://grammateus.ch/config" at "config.xqm";
import module namespace gramm="http://grammateus.ch/gramm" at "gramm.xql";
import module namespace URI="http://grammateus.ch/URI" at "URI.xql";


declare namespace tei = "http://www.tei-c.org/ns/1.0";

(:  
 : OUTDATED MODULE 
 : See Browse Facet module for new search page
 : TODO - improve subtype autocomplete dropdown (have only valid values)
 : 
 :)

(:~
 : Advanced search, build <result> object from a document node.
 :
 : @param $doc - document node
 : @return <result/> object.
 :
:)
declare function search:result ($doc as node()){
        <result uri="{fn:base-uri($doc)}">
            <title>{$doc//tei:title/text()}</title>
            <id>{$doc//tei:idno[@type='TM']}</id>
            <type>{$doc//tei:keywords[@n eq 'text-type']/tei:term/text()}</type>
            <subtype>{$doc//tei:keywords[@n eq 'text-subtype']/tei:term/text()}</subtype>
            <place>{$doc//tei:origPlace/text()}</place>
            <ratio>{$doc//tei:width/text() div $doc//tei:height/text()}</ratio>
        </result>
};

(:~
 : Advanced search, build <results> object from query.
 : order by title
 : fuzzy search => not working. TODO: check why it is returning 3598 when searching Camel??
 :
 : @param $query - a search string to pass to Lucene.
 : @return <results/> object.
 :
:)
declare function search:build-advanced-results ($query as xs:string) as node() {
(:let $q := :)
(:    <query>:)
(:        <fuzzy>{$query}</fuzzy>:)
(:    </query>:)
(:return:)
    <results>{
        for $doc in collection($URI:papyri)//tei:TEI[ft:query(.,$query)] 
        return
            search:result($doc)
    }</results>
};

(:~ Pull out all of the papyri and create a <results/> object.
 :
 : @return <results/> object.
 :
 :)
declare function search:all() as node() {
    <results>{
        for $doc in collection($URI:papyri)
        return
            search:result($doc)
    }
    </results>
};

(:~ Search result message.
 :
 : @param $n - number of hits
 : @return message string.
 :
 :)
declare function search:message ($n as xs:integer) as xs:string {
    if ($n eq 1) 
    then 'There is ' || $n || ' papyrus for:' 
    else 'There are ' || $n || ' papyri for:'
};

(:~ Advanced search, format parameters to HTML
 : 
 :  :)
declare function search:format-parameters ($total as xs:integer, $keyword as xs:string, $type as xs:string, $subtype as xs:string, $shape-h as xs:string, 
            $shape-v as xs:string, $shape-s as xs:string, $fh as xs:string, $fv as xs:string, $width-min, $width-max, $height-min, $height-max)as node()* {
    
    let $parameters := 
        <div class="row">
            <div class="col">
                <div>{search:message($total)}</div>
                <div>
                    <div>{
                        if ($keyword != '') then <a class="mx-1 badge badge-secondary" href="#" role="button">keyword: {$keyword}</a> else (),
                        if ($type != '') then <a class="mx-1 badge badge-secondary" href="#" role="button">type: {$type}</a> else (),
                        if ($subtype != '') then <a class="mx-1 badge badge-secondary" href="#" role="button">subtype: {$subtype}</a> else ()
                    }</div>
                    <div>{
                        if ($shape-h eq 'yes') then <a class="mx-1 badge badge-secondary" href="#">shape: horizontal</a> else (),
                        if ($shape-v eq 'yes') then <a class="mx-1 badge badge-secondary" href="#">shape: vertical</a> else (),
                        if ($shape-s eq 'yes') then <a class="mx-1 badge badge-secondary" href="#">shape: squarish</a> else ()
                    }</div>
                    <div>{
                        if ($fh eq 'yes') then <a class="mx-1 badge badge-secondary" href="#">fibres: horizontal</a> else (),
                        if ($fv eq 'yes') then <a class="mx-1 badge badge-secondary" href="#">fibres: vertical</a> else ()
                    }</div>
                    <div>
                        <a class="mx-1 badge badge-secondary" href="#" role="button">width: {$width-min}-{$width-max}cm</a>
                        <a class="mx-1 badge badge-secondary" href="#" role="button">height: {$height-min}-{$height-max}cm</a>
                    </div>
                </div>
            </div>
            <div class="col d-flex align-items-center justify-content-end">
                <div class="dropdown">
                    <button class="btn btn-default dropdown-toggle mx-1" data-toggle="dropdown" href="#">Sort by
                        <span class="caret"></span>
                    </button>
                    <div class="dropdown-menu" aria-labelledby="dropdownMenuButton">
                        <a class="dropdown-item" href="browse.html?keyword={$keyword}&amp;type={$type}&amp;subtype={$subtype}&amp;width-min={$width-min}&amp;width-max={$width-max}&amp;height-min={$height-min}&amp;height-max={$height-max}&amp;order-by=title">Publication</a>
                        <a class="dropdown-item" href="browse.html?keyword={$keyword}&amp;type={$type}&amp;subtype={$subtype}&amp;width-min={$width-min}&amp;width-max={$width-max}&amp;height-min={$height-min}&amp;height-max={$height-max}&amp;order-by=type">Type</a>
                        <a class="dropdown-item" href="browse.html?keyword={$keyword}&amp;type={$type}&amp;subtype={$subtype}&amp;width-min={$width-min}&amp;width-max={$width-max}&amp;height-min={$height-min}&amp;height-max={$height-max}&amp;order-by=place">Place</a>
                        <a class="dropdown-item" href="browse.html?keyword={$keyword}&amp;type={$type}&amp;subtype={$subtype}&amp;width-min={$width-min}&amp;width-max={$width-max}&amp;height-min={$height-min}&amp;height-max={$height-max}&amp;order-by=TM">TM</a>
                    </div>
                </div>
                <a class="btn btn-default mx-1" href="browse.html">Clear Search {gramm:fa-search()}</a>
            </div>
        </div>
    
    return $parameters
};

(:~ Advanced search, format results to HTML
 : 
 :  :)
declare function search:format-results ($results as node()) {
    <div class="row">
        <div class="col">
            <div id="search-results" class="border-top">{
                for $r in $results/result 
                let $doc := doc($r/@uri/string())
                return
                    <div class="result row border-bottom">
                        <div class="col-md-1"><span class="badge badge-secondary">{$r/id}</span></div>
                        <div class="col-md-11">
                            {(gramm:get-doc-link($doc), if (gramm:get-iiif-manifest($doc)) then gramm:iiif-logo() else ())}
                            <div>{$r/type}</div>
                            <div>{$r/subtype}</div>
                            <span class="px-1">{
                                if (gramm:shape($r/ratio) eq "Vertical") then gramm:fa-shape-vertical()
                                else if (gramm:shape($r/ratio) eq "Horizontal") then gramm:fa-shape-horizontal()
                                else gramm:fa-shape-square()
                            }</span>
                            <span class="px-1">{
                                if (gramm:fibres($doc) eq "Horizontal") then gramm:fa-fibre-horizontal()
                                else gramm:fa-fibre-vertical()
                            }</span>
                        </div>
                    </div>
            }</div>
        </div>
    </div>
};

(:~ Advanced search, entry point
 : 
 : @param $keyword - search string
 : @param $type – document type search string
 : @param $subtype - document subtype search string
 : @param $shape-h – horizontal papyri
 : @param $shape-v – vertical papyri
 : @param $shape-s – square-ish papyri
 : @param $fh – fibres direction horizontal
 : @param $fv – fibres direction vertical
 : @param $width-min - minimum width (double)
 : @param $width-max - maximum width (double)
 : @param $height-min - minimum height (double)
 : @param $height-min - maximum height (double)
 :  :)
declare
%templates:default("keyword", "")
%templates:default("type", "")
%templates:default("subtype", "")
%templates:default("shape-h", "no")
%templates:default("shape-v", "no")
%templates:default("shape-s", "no")
%templates:default("fh", "no")
%templates:default("fv", "no")
%templates:default("order-by", 'title')
%templates:default("width-min", "4")
%templates:default("width-max", "320")
%templates:default("height-min", "3")
%templates:default("height-max", "41")
function search:advanced-search ($node as node (), $model as map (*),
        $keyword as xs:string, $type as xs:string, $subtype as xs:string, $shape-h as xs:string, $shape-v as xs:string,
        $shape-s as xs:string, $fh as xs:string, $fv as xs:string, $order-by as xs:string, 
        $width-min as xs:double, $width-max as xs:double, $height-min as xs:double, $height-max as xs:double) {
        
    (: build results :)
    (: filter docs by keyword, or else search all documents :)
    let $results := 
        if ($keyword != '') then search:build-advanced-results($keyword)
        else search:all()

    (:::: we create the search filters as strings that can be evaluated ::::)

    (: we are filtering <result/> objects :)
    let $results_predicate := "for $result in $results/result "

    (: we need the doc of each result :)
    let $doc_predicate := "let $doc := doc($result/@uri/string()) "
    
    (: return predicate :)
    let $return_predicate := " return if ($doc) then $result else ()"
    
    (: filter 1 - type :)
    let $results_type := 
        if ($type != '') then 
            let $query := concat($results_predicate, $doc_predicate, 'where $doc//tei:keywords[@n eq "text-type"]/tei:term[ft:query(., $type)] ', $return_predicate)
            return <results>{(util:eval($query, fn:true()), util:log('INFO', ($query)))}</results>
        else $results
    
    (: filter 2 - subtype :)
    let $results_subtype := 
        if ($subtype != '') then 
            let $query1 := concat($results_predicate, $doc_predicate, 'where $doc//tei:keywords[@n eq "text-subtype"]/tei:term[ft:query(., $subtype)] ', $return_predicate) 
            return <results>{(util:eval($query1, fn:true()), util:log('INFO', ($query1)))}</results>
        else $results_type
    
    (: filter 3 - shapes :)
    let $results_shapes_all :=
        (: if no shape was selected, return previous results of subtype :)
        if (($shape-h eq 'no' and $shape-v eq 'no' and $shape-s eq 'no'))
        then 
            $results_subtype
        else 
            let $results_predicate := "for $result in $results_subtype/result "
            let $results_sv :=  
                if ($shape-v eq 'yes') then
                    let $query2 := concat($results_predicate, $doc_predicate, 'where gramm:shape(xs:decimal($result/ratio/string())) eq "Vertical" ', $return_predicate) 
                    return (util:eval($query2, fn:true()), util:log('INFO', ($query2)))
                else ()
            let $results_sh :=  
                if ($shape-h eq 'yes') then 
                    let $query := concat($results_predicate, $doc_predicate, 'where gramm:shape(xs:decimal($result/ratio/string())) eq "Horizontal" ', $return_predicate) 
                    return (util:eval($query, fn:true()), util:log('INFO', ($query))) 
                else ()
            let $results_ss := 
                if ($shape-s eq 'yes') then 
                    let $query := concat($results_predicate, $doc_predicate, 'where gramm:shape(xs:decimal($result/ratio/string())) eq "Squarish" ', $return_predicate) 
                    return (util:eval($query, fn:true()), util:log('INFO', ($query))) 
                else ()
        return <results>{($results_sv, $results_sh, $results_ss)}</results>

    (: filter 4 - fibres direction :)
    let $results_fibres_all := 
        (: if no fibres direction was selected, return previous results of shapes :)
        if ($fh eq 'no' and $fv eq 'no') 
        then 
            $results_shapes_all 
        else 
            let $results_predicate := "for $result in $results_shapes_all/result "
            let $results_fh :=  
                if ($fh eq 'yes') then 
                    let $query := concat($results_predicate, $doc_predicate, 'where gramm:fibres($doc) eq "Horizontal" ', $return_predicate ) 
                    return (util:eval($query, fn:true()), util:log('INFO', ($query)))
                else ()
            let $results_fv := 
                if ($fv eq 'yes') then 
                    let $query := concat($results_predicate, $doc_predicate, 'where gramm:fibres($doc) eq "Vertical" ', $return_predicate ) 
                    return (util:eval($query, fn:true()), util:log('INFO', ($query))) 
                else ()
        return <results>{($results_fh, $results_fv)}</results>
    
    (: filter 5 - dimensions :)
    let $results_width := 
        (: if no limitations on dimensions (both min and max values for width/height), return previous results :)
        (: TODO: fix. it is not working! maybe because the slider input has integer values instead of double? :)
        if ($width-min eq $gramm:width_min and $width-max eq $gramm:width_max) 
        then 
            $results_fibres_all 
        else 
            let $results_width :=
                let $results_predicate := "for $result in $results_fibres_all/result "
                let $query := concat($results_predicate, $doc_predicate, 'where xs:double($doc//tei:width/string()) >= $width-min and xs:double($doc//tei:width/string()) <= $width-max ', $return_predicate ) 
                return (util:eval($query, fn:true()), util:log('INFO', ($query)))
        return <results>{$results_width}</results>
        
    let $results_height := 
        (: if no limitations on dimensions (both min and max values for width/height), return previous results :)
        (: TODO: fix. it is not working! maybe because the slider input has integer values instead of double? :)
        if ($height-min eq $gramm:height_min and $height-max eq $gramm:height_max) 
        then 
            $results_width 
        else 
            let $results_height := 
                let $results_predicate := "for $result in $results_width/result "
                let $query := concat($results_predicate, $doc_predicate, 'where xs:double($doc//tei:height/string()) >= $height-min and xs:double($doc//tei:height/string()) <= $height-max ', $return_predicate)
                return (util:eval($query, fn:true()), util:log('INFO', ($query))) 
        return <results>{$results_height}</results>
        
    (: results final - sorted with order-by option :)
    let $results_final := 
    <results>
        {for $r in $results_height/result
        let $doc := doc($r/@uri/string())
        order by 
            if ($order-by eq 'title') then $r/title
            else if ($order-by eq 'type') then $r/type
            else if ($order-by eq 'place') then $r/place
            else if ($order-by eq 'TM') then number($r/id)
            else ()
        return $r}
    </results>
    
    (: count number of hits :)
    let $hits := count($results_final/result)
    
    (: map? :)
    
    (: format results :)
    let $results_html := search:format-results($results_final)
    
    (: format parameters summary :)
    let $params_html := search:format-parameters($hits, $keyword, $type, $subtype, $shape-h, $shape-v, $shape-s, $fh, $fv, $width-min, $width-max, $height-min, $height-max)
    
    (: return results page :)
    return
(:        $results_html:)
        <div class="container-fluid">
            {$params_html}
            <hr/>
            {$results_html}
        </div>
};



(: ---------- TEMPLATE FUNCTIONS ---------- :)

(: create advanced search form :)
(: Form and Results on the same page :)
declare
%templates:default("keyword", "")
%templates:default("type", "")
%templates:default("subtype", "")
%templates:default("shape-h", "no")
%templates:default("shape-v", "no")
%templates:default("shape-s", "no")
%templates:default("fh", "no")
%templates:default("fv", "no")
%templates:default("order-by", 'title')
%templates:default("width-min", "4")
%templates:default("width-max", "320")
%templates:default("height-min", "3")
%templates:default("height-max", "41")
function search:advanced-form($node as node (), $model as map (*),
        $keyword as xs:string, $type as xs:string, $subtype as xs:string, 
        $shape-h as xs:string, $shape-v as xs:string, $shape-s as xs:string, 
        $fh as xs:string, $fv as xs:string, $order-by as xs:string,
        $width-min as xs:double, $width-max as xs:double, $height-min as xs:double, $height-max as xs:double)  {
    
    let $shape-h-value := if ($shape-h eq 'yes') then <input class="form-check-input" type="checkbox" name="shape-h" id="adv_papyrus_horizontal" value="yes" checked=""/>
                             else <input class="form-check-input" type="checkbox" name="shape-h" id="adv_papyrus_horizontal" value="yes"/>

    let $shape-v-value := if ($shape-v eq 'yes') then <input class="form-check-input" type="checkbox" name="shape-v" id="adv_papyrus_vertical" value="yes" checked=""/>
                             else <input class="form-check-input" type="checkbox" name="shape-v" id="adv_papyrus_vertical" value="yes"/>
    
    let $shape-s-value := if ($shape-s eq 'yes') then <input class="form-check-input" type="checkbox" name="shape-s" id="adv_papyrus_square" value="yes" checked=""/>
                             else <input class="form-check-input" type="checkbox" name="shape-s" id="adv_papyrus_square" value="yes"/>

    let $fh-value := if ($fh eq 'yes') then <input class="form-check-input" type="checkbox" name="fh" id="adv_fibres_horizontal" value="yes" checked=""/>
                             else <input class="form-check-input" type="checkbox" name="fh" id="adv_fibres_horizontal" value="yes"/>

    let $fv-value := if ($fv eq 'yes') then <input class="form-check-input" type="checkbox" name="fv" id="adv_fibres_vertical" value="yes" checked=""/>
                             else <input class="form-check-input" type="checkbox" name="fv" id="adv_fibres_vertical" value="yes"/>
    
    return

    <form action="browse.html" method="get" class="form-horizontal">
        <div id="search-form">
            <div class="card">
                <div class="card-header">Keyword
                </div>
                <div class="card-body">
                    <input id="doc-kw" name="keyword" type="text" class="form-control"
                               placeholder="e.g. P. Oxy, Arsinoites, or 44425" value="{$keyword}" />
                </div>
            </div>
            
            <div class="card">
                <div class="card-header">Document Type</div>
                <div class="card-body">
                    <input id="doc-type" name="type" type="text" class="form-control" list="autocomplete_type" autocomplete="off"
                               placeholder="e.g. Declaration" value="{$type}"/>
                    <datalist id="autocomplete_type">
                        {
                            for $category in doc(concat($config:data-root, '/tm_taxonomy.xml'))/tei:taxonomy/tei:category/tei:catDesc
                            let $c := $category/string()
                            return
                                <option value="{$c}"></option>
                        }
                    </datalist>
                </div>
            </div>
            
            <div class="card">
                <div class="card-header">Document Subtype</div>
                <div class="card-body">
                    <input id="doc-subtype" name="subtype" type="text" class="form-control" list="autocomplete_subtype" autocomplete="off"
                               placeholder="e.g. Libellus" value="{$subtype}" />
                    <datalist id="autocomplete_subtype">
                        {
                            let $typology := doc(concat($config:data-root, '/tm_taxonomy.xml'))
                            for $category in distinct-values($typology//tei:category/tei:category/tei:catDesc)
                            let $c := string($category)
                            return
                                <option value="{$c}"></option>
                            }
                        </datalist>
                </div>
            </div>
            
            <div class="card">
                <div class="card-header">Shape</div>
                <div class="card-body">
                    <div class="form-check">
                        {$shape-h-value}
                        <label class="form-check-label">horizontal</label>
                    </div>
                    <div class="form-check">
                        {$shape-v-value}
                        <label class="form-check-label">vertical</label>
                    </div>
                    <div class="form-check">
                        {$shape-s-value}
                        <label class="form-check-label">squarish</label>
                    </div>
                </div>
            </div>
            
            <div class="card">
                <div class="card-header">Fibres Direction</div>
                <div class="card-body">
                    <div class="form-check">
                        {$fh-value}
                        <label class="form-check-label">
                            {gramm:fa-fibre-horizontal()}
                        </label>
                    </div>
                    <div class="form-check">
                        {$fv-value}
                        <label class="form-check-label">
                            {gramm:fa-fibre-vertical()}
                        </label>
                    </div>
                </div>
            </div>
            
            <div class="card">
                <div class="card-header">Dimensions</div>
                <div class="card-body">
                    <label for="width-min">Width:</label>
                    <div class="form-group custom-control-inline">
                        <input type="number" name="width-min" id="width-min" value="{ceiling($width-min)}" class="dimension-input form-control"/>
                        -<input type="number" name="width-max" id="width-max" value="{ceiling($width-max)}" class="dimension-input form-control"/>
                    </div>cm
                    <div id="slider-width"/>
                
                    <br/>
                    <label for = "height-min">Height:</label>
                    <div class="form-group custom-control-inline">
                        <input type="number" name="height-min" id="height-min" value="{ceiling($height-min)}" class="dimension-input form-control"/>
                        -<input type="number" name="height-max" id="height-max" value="{ceiling($height-max)}" class="dimension-input form-control"/>
                    </div>cm
                    <div id="slider-height"/>
                </div>
            </div>
              <script type="text/javascript">
              <![CDATA[ 
                var width_minimum = ]]>{ceiling($gramm:width_min)}<![CDATA[
                var width_maximum = ]]>{ceiling($gramm:width_max)}<![CDATA[
                var height_minimum = ]]>{ceiling($gramm:height_min)}<![CDATA[
                var height_maximum = ]]>{ceiling($gramm:height_max)}<![CDATA[
                    $( function() {
                      $( "#slider-width" ).slider({
                            range: true,
                            min: width_minimum,
                            max: width_maximum,
                            values: [ width_minimum, width_maximum ],
                            slide: function( event, ui ) {
                                $( "#width-min" ).val(ui.values[ 0 ]);
                                $( "#width-max" ).val(ui.values[ 1 ]);
                            }
                      });
                      $( "#slider-height" ).slider({
                            range: true,
                            min: height_minimum,
                            max: height_maximum,
                            values: [ height_minimum, height_maximum ],
                            slide: function( event, ui ) {
                                $("#height-min").val(ui.values[ 0 ]);
                                $("#height-max").val(ui.values[ 1 ]);
                            }
                      });
                      //$("#width-min").val($("#slider-width").slider( "values", 0 ));
                      //$("#width-max").val($("#slider-width").slider( "values", 1 ));
                      //$("#height-min").val($("#slider-height").slider( "values", 0 ));
                      //$("#height-max").val($("#slider-height").slider( "values", 1 ));
                    });
                ]]>
                  
              </script>
            
        </div>    
        <div class="form-row">
            <div class="col-sm-12">
                <input class="btn btn-danger" type="submit" value="Search"/>
            </div>
        </div>
        
    </form>
};
