<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:tei="http://www.tei-c.org/ns/1.0" xmlns:map="http://www.w3.org/2005/xpath-functions/map" xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="#all" version="3.0">
    

<xsl:param name="sections" select="//tei:ab[@type eq 'section']"/>
  
  <xsl:variable name="classes-per-line" as="map(xs:integer, xs:string*)" select="map:merge(for $section in $sections/tei:locus, $line in $section/@from to $section/@to return map { $line : substring-after($section/@ana/string(), '#') }, map { 'duplicates' : 'combine' })"/>

  <xsl:mode on-no-match="shallow-copy"/>

  <xsl:output method="html" indent="yes" html-version="5"/>
  
  <xsl:template match="/">
      <xsl:value-of select="serialize($classes-per-line, map { 'method' : 'adaptive' })"/>
      <xsl:iterate select="sort(map:keys($classes-per-line))">
          <xsl:param name="html" select="."/>
          <xsl:on-completion select="$html"/>
          <xsl:next-iteration>
              <xsl:with-param name="html">
                  <xsl:apply-templates select="$html" mode="wrap">
                      <xsl:with-param name="line" tunnel="yes" select="."/>
                  </xsl:apply-templates>                  
              </xsl:with-param>
          </xsl:next-iteration>
      </xsl:iterate>
  </xsl:template>
  
  <xsl:mode name="wrap" on-no-match="shallow-copy"/>
  
  <xsl:template match="*[*/@id]" mode="wrap">
      <xsl:param name="line" tunnel="yes"/>
      <xsl:choose>
          <xsl:when test="(*/@id = 'l' || $line) or (*/@id = 'ms-l' || $line)">
          <!-- */@id = 'ar-l' || $line 
                contains(*/@id, concat('l', $line)) and matches(*/@id, '[^r]*')-->
              <xsl:copy>
                  <xsl:apply-templates select="@*"/>
                  <xsl:for-each-group select="node()" group-starting-with="*[@id]">
                      <xsl:choose>
                          <xsl:when test="self::*[@id = 'l' || $line] or self::*[@id = 'ms-l' || $line]">
                              <xsl:apply-templates select="." mode="#current"/>
                              <span class="section {$classes-per-line($line)}">
                                  <xsl:apply-templates select="tail(current-group())" mode="#current"/>
                              </span>
                          </xsl:when>
                          <xsl:otherwise>
                                <xsl:apply-templates select="current-group()" mode="#current"/>
                          </xsl:otherwise>
                      </xsl:choose>
                  </xsl:for-each-group>
              </xsl:copy>
          </xsl:when>
          <!-- not very good: for TM 19654, need to enclose lines that are not in the sections into span as well
                here it is enclosing in twice... 
                and in 9270, line 0 is not in a span
                 -->
<!--          <xsl:when test="*/@id">-->
<!--              <xsl:copy>-->
<!--                  <xsl:apply-templates select="@*"/>-->
<!--                  <xsl:for-each-group select="node()" group-starting-with="*[@id]">-->
<!--                      <xsl:choose>-->
<!--                          <xsl:when test="(self::*[starts-with(@id, 'l')] or self::*[starts-with(@id, 'ms-l')]) and self::*[following-sibling::*[text()]]">-->
<!--                              <xsl:apply-templates select="." mode="#current"/>-->
<!--                              <span class="section">-->
<!--                                  <xsl:apply-templates select="tail(current-group())" mode="#current"/>-->
<!--                              </span>-->
<!--                          </xsl:when>-->
<!--                          <xsl:otherwise>-->
<!--                                <xsl:apply-templates select="current-group()" mode="#current"/>-->
<!--                          </xsl:otherwise>-->
<!--                      </xsl:choose>-->
<!--                  </xsl:for-each-group>-->
<!--              </xsl:copy>-->
<!--          </xsl:when>-->
          <xsl:otherwise>
              <xsl:next-match/>
          </xsl:otherwise>
      </xsl:choose>
  </xsl:template>
  
</xsl:stylesheet>