# Grammateus App

## Contents
1. [Introduction](#introduction)
2. [App Content](#app-content)
   * [Data](#data)
   * [Modules](#modules)
   * [Resources](#resources)
   * [Templates](#templates)


## Introduction

### eXist
[eXist](http://exist-db.org) is an Open Source platform for XML database and application. It makes use of XQuery, a standardized language to query XML documents.

Installation instructions:
https://exist-db.org/exist/apps/doc/basic-installation

Running version: eXist 5.2.0.


### App Creation
Instructions on how to create an app are available here: <https://exist-db.org/exist/apps/doc/development-starter>. In addition, this tutorial is a helpful resource to get started with app development in eXist: <https://howto.acdh.oeaw.ac.at/blog/books/how-to-build-a-digital-edition-web-app/>.

We have created an empty app from the Plain template, using Boostrap CSS. The app was populated with the contents described below.

We have added the `$app-root` variable in the [controller.xql](../app/controller.xql) file:
```XQuery
(: Resource paths starting with $app-root are loaded from the application's root collection :)
else if (contains($exist:path,"$app-root")) then
<dispatch xmlns="http://exist.sourceforge.net/NS/exist">
    <forward url="{$exist:controller}/{substring-after($exist:path, '$app-root/')}">
        <set-header name="Cache-Control" value="no"/>
    </forward>
</dispatch>
 ```

We have also removed all links to shared resources (`$shared`), and replaced them with links to local app [resources](../app/resources).


## App Content
In the app [main folder](../app) are located the HTML pages of the website. There you also find configuration files: `build.xml`, `expath-pkg.xml` and `repo.xml`.

There are two important scripts in the main folder:
- [database-update-data.xq](../app/database-update-data.xq), which loads new or updated data from a local folder into the database.
- [database-update-HGV.xq](../app/database-update-HGV.xq), which fetches the place and date of origin from HGV data on papyri.info. It is necessary to store this data in our database for indexing purposes, even if for each papyrus we display the most up-to-date information dynamically from papyri.info.

The rest of the app's content is divided into Data, Modules, Resources and Templates.

### Data
In the Data folder are stored all the TEI XML files:
  - bibliographic references created from our [zotero group](https://www.zotero.org/groups/2288077/grammateus) with [zotero2bibl](https://gitlab.unige.ch/papyri-dev/zotero2bibl).
  - texts for the introduction, descriptions of types of papyri and explanation of the classification of papyri into types of documents.
  - papyri (one XML file per papyrus in the database), see also the [data documentation](documentation-grammateus-data.md) for more information on the creation and contents of these files.
  - the [grammateus typology](../app/data/grammateus_taxonomy.xml).
  - [authority lists](../app/data/authority_lists.xml).
  - links to external resources such as texts and images.

### Modules
The `config.xqm` and `view.xql` modules come by default with the eXist app. Modules must be added to the `view.xql` for the controller to apply HTML [templating](https://exist-db.org/exist/apps/doc/templating). The other modules were created specifically for grammateus.  

**app**

The app module contains most templating function for the website, for instance the about page, the introduction, the typology page with descriptions and classification. This module deals with functions of the bibliography, and the page for displaying results from a general search using the searchbox on the top right corner of the website. It also sets the page title, a function that is called from the [template](#templates) HTML page.

**search-facets**

This module contains the template for the `papyri.html` page and all functions related to searching the papyri in the database. 

**show**

This modules has the template and function for displaying a single papyrus with metadata, a representation of the papyrus in SVG, and eventually an image displayed with Mirador for the papyri that are available with IIIF, e.g. TM 12902.

Among the metadata, the Provenance and Century are accessed directly from papyri.info, so as to have the most up-to-date information.

The `show:svg` function for [SVG display](https://gitlab.unige.ch/papyri-dev/grammateus/-/blob/9102c0ff707cca5988a6d70591ad7835f508df81/app/modules/show.xql#L225-314) is the most complex part of the page, and the one constantly raising new problems, as it is an important project requirement to access the text dynamically from papyri.info. Because of that requirement, we cannot store a normalized version of the text, but have to deal with the relatively high level of variation in encodings.

**gramm**

In the gramm module there are functions to use across the other modules. For instance:
- getting general information about a papyrus [:link:](https://gitlab.unige.ch/papyri-dev/grammateus/-/blob/9102c0ff707cca5988a6d70591ad7835f508df81/app/modules/gramm.xql#L51-87)
- getting external data, from papyri.info [:link:](https://gitlab.unige.ch/papyri-dev/grammateus/-/blob/9102c0ff707cca5988a6d70591ad7835f508df81/app/modules/gramm.xql#L148-251)
- font awesome icons [:link:](https://gitlab.unige.ch/papyri-dev/grammateus/-/blob/9102c0ff707cca5988a6d70591ad7835f508df81/app/modules/gramm.xql#L89-146)
- place, shape and fibres [:link:](https://gitlab.unige.ch/papyri-dev/grammateus/-/blob/9102c0ff707cca5988a6d70591ad7835f508df81/app/modules/gramm.xql#L254-331)
- date functions [:link:](https://gitlab.unige.ch/papyri-dev/grammateus/-/blob/9102c0ff707cca5988a6d70591ad7835f508df81/app/modules/gramm.xql#L333-421)
- formatting the general search results [:link:](https://gitlab.unige.ch/papyri-dev/grammateus/-/blob/9102c0ff707cca5988a6d70591ad7835f508df81/app/modules/gramm.xql#L424-461)
- functions related to the typology [:link:](https://gitlab.unige.ch/papyri-dev/grammateus/-/blob/9102c0ff707cca5988a6d70591ad7835f508df81/app/modules/gramm.xql#L464-542)

**URI**

THe URI module gathers various URIs, either internal to the database or external (papyri.info, HGV).

### Resources

**css**

There is the bootstrap css, `nouislider.css` for the date sliders on the papyri.html page, plus a custom `style.css` file.

**js**

We have bootstrap and jquery javascript, nouislider for the date sliders, and svg-pan-zoom for the papyrus display in svg. `grammateus.js` has custom javascript, in particular to solve problems with the display of several papyri, as some encoding difficulties prevent us from fully automating the creation of the display (e.g. we mostly display the recto of papyri and hide the verso text, but in a couple of cases we need to display the verso).

It should be noted that svg-pan-zoom is currently making the display too small for some papyri on Chrome and Edge. An alternative solution should be found (see [issue](#64)).

**fonts**

We are using fontawesome for the icons.

**xml**

Here we have the lucene index configuration of the papyri folder. We keep it here for convenience, when updating the website on server.

**xslt**

The EpiDoc XSLT Stylesheet are used to convert the TEI transcriptions from papyri.info to an HTML display. The main change that we have included is how the lines are numbered, so as to be able to display the text sections (see [htm-teilb.xsl](https://gitlab.unige.ch/papyri-dev/grammateus/-/blob/9102c0ff707cca5988a6d70591ad7835f508df81/app/resources/xslt/epidoc/htm-teilb.xsl)). Another XSLT, [txt-sect.xslt](https://gitlab.unige.ch/papyri-dev/grammateus/-/blob/9102c0ff707cca5988a6d70591ad7835f508df81/app/resources/xslt/txt-sect.xslt), adds the text section to the HTML transcription.

`typologyToHTML` converts an XML TEI description into HTML, and `taxonomyToList` creates the typology display on the [typology page](https://grammateus.unige.ch/typology.html).

Finally, `sidebar.xslt` creates the sidebar navigation for the description and classification pages.

**mirador**

We use the mirador viewer to display images available in IIIF. We are using mirador 2.0 and should upgrade to the latest version.

**img**

We keep a few logos and the three papyri images for the home page.

### Templates
There is a single page template, that sets the metadata, header and footer for every page on the website.



